@@ -1,4 +1,4 @@
-/*  Copyright (c) 2003-2013 Xfce Development Team
+/*  Copyright (c) 2003-2014 Xfce Development Team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -305,6 +305,30 @@ icon_theme_load(const gchar *dir)
 }
 
 
+/*
+ * Compare two icon_theme structs using their path names, returning
+ * the result as a qsort()-style comparison function (less than zero
+ * for first arg is less than second arg, zero for equal, greater zero
+ * if first arg is greater than second arg).
+ */
+static gint
+icon_theme_compare(gconstpointer a,
+                   gconstpointer b)
+{
+    icon_theme *it1 = *(icon_theme **) a;
+    icon_theme *it2 = *(icon_theme **) b;
+
+    if (G_UNLIKELY(it1 == NULL && it2 == NULL))
+        return 0;
+    if (G_UNLIKELY(it1 == NULL))
+        return -1;
+    if (G_UNLIKELY(it2 == NULL))
+        return 1;
+
+    return g_strcmp0(it1->dir, it2->dir);
+}
+
+
 static GArray *
 find_themes_in_dir(const gchar *path)
 {
@@ -339,10 +363,24 @@ find_themes_in_dir(const gchar *path)
         weather_debug("Found %d icon theme(s) in %s.", themes->len, path);
     } else
         weather_debug("Could not list directory %s.", path);
+    g_array_sort(themes, (GCompareFunc) icon_theme_compare);
     return themes;
 }
 
 
+/*
+ * Returns the user icon theme directory as a string which needs to be
+ * freed by the calling function.
+ */
+gchar *
+get_user_icons_dir(void)
+{
+    return g_strconcat(g_get_user_config_dir(), G_DIR_SEPARATOR_S,
+                       "xfce4", G_DIR_SEPARATOR_S, "weather",
+                       G_DIR_SEPARATOR_S, "icons", NULL);
+}
+
+
 /*
  * Find all available themes in user's config dir and at the install
  * location.
@@ -356,9 +394,7 @@ find_icon_themes(void)
     themes = g_array_new(FALSE, TRUE, sizeof(icon_theme *));
 
     /* look in user directory first */
-    dir = g_strconcat(g_get_user_config_dir(), G_DIR_SEPARATOR_S,
-                      "xfce4", G_DIR_SEPARATOR_S, "weather",
-                      G_DIR_SEPARATOR_S, "icons", NULL);
+    dir = get_user_icons_dir();
     found = find_themes_in_dir(dir);
     g_free(dir);
     if (found) {
